package xyz.twovb.sgm.games.impl.capturethebrick;

/*
 * Created by 2vb - 3/7/2024
 */

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.command.CommandSender;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.HandlerList;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.PlayerMoveEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import xyz.twovb.sgm.SGM;
import xyz.twovb.sgm.games.GameManager;
import xyz.twovb.sgm.games.Minigame;
import xyz.twovb.sgm.levels.LevelManager;
import xyz.twovb.toolbox.api.CustomPlayer;
import xyz.twovb.toolbox.utils.ChatUtils;
import xyz.twovb.toolbox.utils.CustomLogger;

import java.io.File;
import java.io.IOException;
import java.util.*;

public class CTB implements Minigame {

    final static String name = "CaptureTheBrick";
    private final UUID uuid;
    private final List<Player> players;
    private final ArrayList<Location> redBrickSpawns = new ArrayList<Location>();
    private final ArrayList<Location> blueBrickSpawns = new ArrayList<Location>();
    private ArrayList<Location> redSpawnArea = new ArrayList<Location>();
    private ArrayList<Location> blueSpawnArea = new ArrayList<Location>();
    private World gameWorld;
    private GameState state;
    private int bricks;
    private int territoryLevel;
    private int boundsLevel;

    public CTB() {
        this.uuid = UUID.randomUUID();
        this.state = GameState.PRESTART;
        this.players = new ArrayList<>();
    }

    public void init(CommandSender owner, String world) throws IOException {
        // Perform initialization logic here
        Bukkit.getPluginManager().registerEvents(this, SGM.getInstance());
        gameWorld = createGameWorld(world, uuid);
        gameWorld.setAutoSave(false);
        try {
            if (applyOptions()) {
                state = GameState.READY;
                owner.sendMessage("A new " + name + " instance with ID " + uuid + " has been created and is now ready!");
            }
        } catch (Exception ex) {
            owner.sendMessage(ChatUtils.translate(SGM.getInstance().getMessages().getString("sgm.game.cant-start")));
            SGM.getInstance().getCLogger().error(ex.getMessage());
        }
    }

    private Location parseString(String string) {
        String[] coords = string.trim().split(",");
        if (coords.length == 3) {
            return new Location(gameWorld, Integer.parseInt(coords[0]), Integer.parseInt(coords[1]), Integer.parseInt(coords[2]));
        } else {
            return null;
        }
    }

    // Attempts to load the game's options as a YamlConfiguration
    // before processing each section in the same order as the file.
    private boolean applyOptions() {
        File configFile = new File(LevelManager.gamePath + name.toLowerCase() + "/options.yml");
        YamlConfiguration config = YamlConfiguration.loadConfiguration(configFile);

        // Set map information
        ConfigurationSection mapInfoSection = config.getConfigurationSection("mapInfo");
        if (mapInfoSection != null) {
            this.bricks = mapInfoSection.getInt("brickCount");
            this.territoryLevel = mapInfoSection.getInt("territory");
            this.boundsLevel = mapInfoSection.getInt("boundary");
        } else {
            return false;
        }

        // Process team information
        List<String> teamsToProcess = Arrays.asList("red", "blue");

        for (String teamName : teamsToProcess) {
            ConfigurationSection team = config.getConfigurationSection("teams." + teamName);
            if (team != null) {
                List<String> brickSpawns = team.getStringList("brickSpawns");
                List<String> playerSpawnArea = team.getStringList("playerSpawnArea");

                // Get brick locations
                if (brickSpawns.size() >= bricks) {
                    for (String rawBrickLoc : brickSpawns) {
                        Location brickLoc = parseString(rawBrickLoc);
                        if (teamName.equals("red")) {
                            this.redBrickSpawns.add(brickLoc);
                        } else if (teamName.equals("blue")) {
                            this.blueBrickSpawns.add(brickLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid brick spawns. Please fix before trying again.");
                    return false;
                }

                // Hopefully that works!
                if (playerSpawnArea.size() == 2) {
                    for (String rawSpawnLoc : playerSpawnArea) {
                        Location spawnLoc = parseString(rawSpawnLoc);
                        if (teamName.equals("red")) {
                            this.redSpawnArea.add(spawnLoc);
                        } else if (teamName.equals("blue")) {
                            this.blueSpawnArea.add(spawnLoc);
                        }
                    }
                } else {
                    SGM.getInstance().getCLogger().error("Invalid spawn area. Please fix before trying again.");
                    return false;
                }
            } else {
                return false;
            }
        }
        return true;
    }

    private void placeBrick(Location location, String team) {
        Block brick = location.getBlock();
        if (Objects.equals(team, "red")) {
            brick.setType(Material.RED_WOOL);
        }
        if (Objects.equals(team, "blue")) {
            brick.setType(Material.BLUE_WOOL);
        }
        SGM.getInstance().getCLogger().log("Placed " + brick.getType() + " at " + location);
    }

    @Override
    public void start() {
        if (state == GameState.READY) {
            state = GameState.STARTED;
            sendMessageToAllPlayers("The game has started!");
            for (Player player : players) {
                player.teleport(player.getWorld().getSpawnLocation());
            }
            Bukkit.getScheduler().runTaskTimer(SGM.getInstance(), this::onTick, 0L, 1L);
        }
    }

    @Override
    public void stop() {
        // Stop game logic
        state = GameState.ENDING;
        Player winner;
        if (players.size() <= 1) {
            winner = players.get(0);
            winner.sendMessage("Congratulations on winning the game!");
        }
        players.clear(); // Clear player list
        GameManager.getActiveGames().remove(uuid);
        HandlerList.unregisterAll(this);
    }

    @Override
    public void addPlayer(Player player) {
        players.add(player);
        player.teleport(gameWorld.getSpawnLocation());
        player.teleport(new Location(gameWorld, 0, 0, 0));
        sendMessageToAllPlayers(player.getName() + " has joined the game!");
    }

    @Override
    public void removePlayer(Player player) {
        players.remove(player);
    }

    @EventHandler
    void onPlayerQuit(PlayerQuitEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player)) {
            removePlayer(event.getPlayer());
        }
    }

    @EventHandler
    void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getPlayer();
        if (players.contains(player) && state == GameState.STARTED) {
            sendMessageToAllPlayers(player.getName() + " was eliminated!");
            event.setDeathMessage("");
            removePlayer(player);
        }
    }

    @Override
    public List<Player> getPlayers() {
        return players;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public UUID getGameId() {
        return uuid;
    }

    @Override
    public GameState getState() {
        return state;
    }

    @EventHandler
    void onPlayerMove(PlayerMoveEvent event) {
        Player player = event.getPlayer();
        CustomPlayer cPlayer = new CustomPlayer(player);
        cPlayer.sendMessage("move");
        if (event.getFrom().getX() != event.getTo().getX() || event.getFrom().getZ() != event.getTo().getZ()) {
            cPlayer.sendMessage("move full block");
        }
    }

    @Override
    public void onTick() {
        if (state == GameState.STARTED && players.size() <= 1) {
            this.stop();
        }
    }

}